#!/bin/bash -ex
# write stdout/stderr to the oom.log file
exec &>>/app/logs/error-exitcode.log

CODE=$1
WHEN=$(date)
echo "Error exitcode ${CODE} at ${WHEN}"

if [ $# -eq 0 ]; then
    # If no arguments provided, it was invoked by the JVM due to the
    ##  -XX:OnOutOfMemoryError flag. This will actually trigger this
    ##  script to be invoked a second time due to the JVM _actually_
    ##  dying and elasticsearch.sh completing it's execution.
    ##  The second invocation will have the exit code 137.
    if [[ $(type -P killall) ]]; then
        killall -9 java
    else
        pkill -9 java
    fi
    exit 0
fi

handle_error_code() {
    local extra_args=${1}
    {% if useWipeInstanceState|default(true) %}
        {% set configPath = '/app/config/elasticsearch.yml' if usingFoundPlugin|default(true) else '/app/config/wipe-instance-state.yml' %}
        /usr/local/bin/wipe-instance-state --config {{ configPath }} --script-timeout 12000 ${extra_args}
    {% else %}
    # write json file to /app/restart_events with slain value
    slain=false
    for var in $extra_args
    do
        case $var in --slain)
                slain=true
                ;;
        esac

    done
    file_timestamp=$(date '+%Y%m%d%H%M%S')
    timestamp=$(date +%s%N | cut -b1-13) # this will give milliseconds since epoch
    echo "{\"timestamp\":${timestamp},\"exit_code\":${CODE},\"slain\":${slain}}" >> /app/restart_events/restart_event_${file_timestamp}.json
    {% endif %}
    echo "slain at ${WHEN} with ${CODE}" >> /app/logs/slain.log
}

case ${CODE} in
      1)  # ElasticsearchUncaughtExceptionHandler: Unknown fatal error
        handle_error_code
        ;;
      3)  # FileAlreadyExistsException - GH issue #3831
        handle_error_code
        ;;
     78)  # Bootstrap check failure - GH issue #3831
        handle_error_code
        ;;
    124)  # ElasticsearchUncaughtExceptionHandler: Serious I/O error
        handle_error_code
        ;;
    125)  # ElasticsearchUncaughtExceptionHandler: Unknown virtual machine error
        handle_error_code
        ;;
    126)  # ElasticsearchUncaughtExceptionHandler: Stack overflow error
        handle_error_code
        ;;
    127)  # ElasticsearchUncaughtExceptionHandler: Out of memory error
        handle_error_code --slain
        ;;
    128)  # ElasticsearchUncaughtExceptionHandler: VM internal error
        handle_error_code
        ;;
    158)  # 128 + 30 - killed by jvmkiller agent
        handle_error_code --slain
        ;;
    137)  # 128 + 9 - killed by kernel oom-killer
        handle_error_code --slain
        ;;
    134)  # 128 + 6 - segfault
        handle_error_code
        ;;
    *)
        echo "No idea what to do with error code $CODE"
        ;;
esac

HEAP_DUMPS=$(find /app/heap_dumps/ -maxdepth 1 -type f -regex '.*\.hprof')
if [ -n "$HEAP_DUMPS" ]; then
    echo -n "Compressing heap dumps "
    if [[ $(type -P brotli) ]]; then
        # Brotli is preferred when available because it has a better speed to compression ratio.
        echo "with brotli"
        echo "$HEAP_DUMPS" | xargs brotli --rm
        mv /app/heap_dumps/*.hprof.br /app/heap_dumps/compressed/
    elif [[ $(type -P pigz) ]]; then
        # pigz creates regular gz archives, but supports utilizing multiple cores
        echo "with pigz"
        echo "$HEAP_DUMPS" | xargs pigz -p `expr $(nproc) / 2`
        mv /app/heap_dumps/*.hprof.gz /app/heap_dumps/compressed/
    else
        echo "with gzip"
        echo "$HEAP_DUMPS" | xargs gzip
        mv /app/heap_dumps/*.hprof.gz /app/heap_dumps/compressed/
    fi
fi


echo "exiting on-error-exitcode.sh at $(date)"
exit 0
